@@ -22,14 +22,17 @@
 #define	DIST_FUNC		"functions"
 #define	DIST_SCRIPTS		"scripts"
 
-#define	ADD_IP			1
-#define	DEL_IP			2
-#define	SET_HOSTNAME		3
-#define	SET_DNS			4
-#define	SET_USERPASS		5
-#define	SET_UGID_QUOTA		6
-#define	POST_CREATE		7
-#define	PRE_START		8
+enum {
+	ADD_IP = 1,
+	DEL_IP,
+	SET_HOSTNAME,
+	SET_DNS,
+	SET_USERPASS,
+	SET_UGID_QUOTA,
+	POST_CREATE,
+	PRE_START,
+	SET_CONSOLE,
+};
 
 typedef struct {
 	char *def_ostmpl;
@@ -48,6 +51,7 @@ typedef struct dist_actions {
 	char *set_ugid_quota;	/**< setup 2level quota. */
 	char *post_create;	/**< sostcreate actions. */
 	char *pre_start;	/**< pre start actions. */
+	char *set_console;	/**< for vzctl console. */
 } dist_actions;
 
 /* Read distribution specific actions configuration file.
