@@ -226,7 +226,6 @@ void fill_container_param(struct arg_start *arg,
 	create_param->iptables_mask = get_ipt_mask(&arg->res->env);
 	logger(3, 0, "Setting iptables mask %#10.8llx",
 			(unsigned long long) create_param->iptables_mask);
-	clean_hardlink_dir("/");
 	if (arg->res->cpu.vcpus != NULL)
 		create_param->total_vcpus = *arg->res->cpu.vcpus;
 
@@ -268,6 +267,10 @@ int exec_container_init(struct arg_start *arg,
 		}
 	}
 
+	if (mkdir("/proc", 0555) && errno != EEXIST)
+		return vzctl_err(VZ_SYSTEM_ERROR, errno,
+				"Can't mkdir /proc");
+
 	if (statfs("/proc", &sfs))
 		return vzctl_err(VZ_SYSTEM_ERROR, errno,
 				"statfs on /proc failed");
@@ -306,6 +309,9 @@ int exec_container_init(struct arg_start *arg,
 	}
 
 	logger(10, 0, "Starting init");
+
+	close_fds(0, arg->err_p, -1);
+
 	execve("/sbin/init", argv, envp);
 	execve("/etc/init", argv, envp);
 	execve("/bin/init", argv, envp);
@@ -657,6 +663,11 @@ int vps_start_custom(vps_handler *h, envid_t veid, vps_param *param,
 			quota_inc(&res->dq, -100);
 	}
 
+	if (!(skip & SKIP_ACTION_SCRIPT)) {
+		ret = run_pre_script(veid, VPS_PRESTART);
+		if (ret)
+			return ret;
+	}
 	if ((ret = fill_vswap_ub(&res->ub, &res->ub)))
 		return ret;
 
